/*
 * modbus-tcp - context.h
 *	version 0.1.1
 *
 * Copyright (C) 2006..2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CONTEXT_H
#define __CONTEXT_H

#include <list.h>
#include <inttypes.h>
#include <stdlib.h>
#include <modbus.h>
#include <buffer.h>

struct c_request;
struct c_context;

struct c_request {
	struct modbus_request mreq;
	unsigned int tcp_id;		/* ModBUS TCP request id */
	unsigned int tcp_addr;		/* ModBUS TCP device id */
	struct c_context *ctx;		/* context, NULL for orphans */
	unsigned int sent;
	struct list_head list;
};

/*
 * c_context - describes client context
 */
struct c_context {
	int fd;				/* file descriptor */
	struct buffer receive;		/* request receive buffer */
	struct buffer receive_hdr;	/* request receive header buffer */
	struct c_request *send;		/* request to send */
	struct list_head req_ready;	/* list of completed requests */
	struct list_head req_wait;	/* list of waiting requests */
	unsigned int rcount;			/* number of requests */
	struct list_head list;		/* for lists of contexts */
	int write_wait;
};

int context_init(struct c_context *c, int fd);
int context_destroy(struct c_context *c);
int context_read(struct c_context *c);
int context_write(struct c_context *c);
struct c_request *c_curr(struct c_context *c);
void orphan_killer(void);

void c_req_add(struct c_context *c, struct c_request *r);
int c_req_completed(struct c_request *cr);
void request_destroy(struct c_request *r);

#endif
