/*
 * rejd - rejd_byteorder.h
 * 	version 0.1.6
 *
 * 	rejd byteorder
 * 
 * Copyright (C) 2005  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef REJD_BYTEORDER_INCLUDED
#define REJD_BYTEORDER_INCLUDED

#ifdef HAVE_ENDIAN_H
#include <endian.h>
#endif

#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#endif

#if BYTE_ORDER == BIG_ENDIAN

#ifndef __BIG_ENDIAN_BITFIELD
#define __BIG_ENDIAN_BITFIELD
#endif

#define cpu_to_be16(x)		(x)
#define cpu_to_be32(x)		(x)
#define cpu_to_be64(x)		(x)
#define cpu_to_le16(x)		(bswap_16(x))
#define cpu_to_le32(x)		(bswap_32(x))
#define cpu_to_le64(x)		(bswap_64(x))

#define be16_to_cpu(x)		(x)
#define be32_to_cpu(x)		(x)
#define be64_to_cpu(x)		(x)
#define le16_to_cpu(x)		(bswap_16(x))
#define le32_to_cpu(x)		(bswap_32(x))
#define le64_to_cpu(x)		(bswap_64(x))

#else

#if BYTE_ORDER == LITTLE_ENDIAN

#ifndef __LITTLE_ENDIAN_BITFIELD
#define __LITTLE_ENDIAN_BITFIELD
#endif

#define cpu_to_be16(x)		(bswap_16(x))
#define cpu_to_be32(x)		(bswap_32(x))
#define cpu_to_be64(x)		(bswap_64(x))
#define cpu_to_le16(x)		(x)
#define cpu_to_le32(x)		(x)
#define cpu_to_le64(x)		(x)

#define be16_to_cpu(x)		(bswap_16(x))
#define be32_to_cpu(x)		(bswap_32(x))
#define be64_to_cpu(x)		(bswap_64(x))
#define le16_to_cpu(x)		(x)
#define le32_to_cpu(x)		(x)
#define le64_to_cpu(x)		(x)

#else

#error "Only little and big endian systems are supported."

#endif
#endif

#endif
